Package: ritis
Title: Integrated Taxonomic Information System Client
Description: An interface to the Integrated Taxonomic Information System ('ITIS')
    (<https://www.itis.gov>). Includes functions to work with the 'ITIS' REST
    API methods (<https://www.itis.gov/ws_description.html>), as well as the
    'Solr' web service (<https://www.itis.gov/solr_documentation.html>).
Version: 1.0.0
License: MIT + file LICENSE
URL: https://github.com/ropensci/ritis (devel)
        https://docs.ropensci.org/ritis/ (docs)
BugReports: https://github.com/ropensci/ritis/issues
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"),
        email = "myrmecocystus@gmail.com",
        comment = c(ORCID = "0000-0003-1444-9135")),
    person(given = "rOpenSci",
        role = "fnd",
        comment = "https://ropensci.org/"))
LazyData: true
Encoding: UTF-8
Language: en-US
Imports: solrium (>= 1.1.4), crul (>= 0.9.0), jsonlite, data.table,
        tibble
Suggests: testthat, webmockr, vcr (>= 0.5.4)
RoxygenNote: 7.1.1
X-schema.org-applicationCategory: Taxonomy
X-schema.org-keywords: taxonomy, biology, nomenclature, JSON, API, web,
        api-client, identifiers, species, names
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2021-02-02 06:21:27 UTC; sckott
Author: Scott Chamberlain [aut, cre] (<https://orcid.org/0000-0003-1444-9135>),
  rOpenSci [fnd] (https://ropensci.org/)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2021-02-02 07:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 10:11:16 UTC; unix

Package: ritis
Title: Integrated Taxonomic Information System Client
Description: An interface to the Integrated Taxonomic Information System ('ITIS')
    (<https://www.itis.gov>). Includes functions to work with the 'ITIS' REST
    API methods (<https://www.itis.gov/ws_description.html>), as well as the
    'Solr' web service (<https://www.itis.gov/solr_documentation.html>).
Version: 1.0.0
License: MIT + file LICENSE
URL: https://github.com/ropensci/ritis (devel)
        https://docs.ropensci.org/ritis/ (docs)
BugReports: https://github.com/ropensci/ritis/issues
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"),
        email = "myrmecocystus@gmail.com",
        comment = c(ORCID = "0000-0003-1444-9135")),
    person(given = "rOpenSci",
        role = "fnd",
        comment = "https://ropensci.org/"))
LazyData: true
Encoding: UTF-8
Language: en-US
Imports: solrium (>= 1.1.4), crul (>= 0.9.0), jsonlite, data.table,
        tibble
Suggests: testthat, webmockr, vcr (>= 0.5.4)
RoxygenNote: 7.1.1
X-schema.org-applicationCategory: Taxonomy
X-schema.org-keywords: taxonomy, biology, nomenclature, JSON, API, web,
        api-client, identifiers, species, names
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2021-02-02 06:21:27 UTC; sckott
Author: Scott Chamberlain [aut, cre] (<https://orcid.org/0000-0003-1444-9135>),
  rOpenSci [fnd] (https://ropensci.org/)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2021-02-02 07:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(accepted_names)
export(any_match_count)
export(comment_detail)
export(common_names)
export(core_metadata)
export(coverage)
export(credibility_rating)
export(credibility_ratings)
export(currency)
export(date_data)
export(description)
export(experts)
export(full_record)
export(geographic_divisions)
export(geographic_values)
export(global_species_completeness)
export(hierarchy_down)
export(hierarchy_full)
export(hierarchy_up)
export(itis_facet)
export(itis_group)
export(itis_highlight)
export(itis_search)
export(jurisdiction_origin_values)
export(jurisdiction_values)
export(jurisdictional_origin)
export(kingdom_name)
export(kingdom_names)
export(last_change_date)
export(lsid2tsn)
export(other_sources)
export(parent_tsn)
export(publications)
export(rank_name)
export(rank_names)
export(record)
export(review_year)
export(scientific_name)
export(search_any_match_paged)
export(search_anymatch)
export(search_common)
export(search_scientific)
export(synonym_names)
export(taxon_authorship)
export(terms)
export(tsn2lsid)
export(tsn_by_vernacular_language)
export(unacceptability_reason)
export(usage)
export(vernacular_languages)

ritis
=====



[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![cran checks](https://cranchecks.info/badges/worst/ritis)](https://cranchecks.info/pkgs/ritis)
[![Build Status](https://travis-ci.org/ropensci/ritis.svg?branch=master)](https://travis-ci.org/ropensci/ritis)
[![Build status](https://ci.appveyor.com/api/projects/status/pvrc9muevha00fie/branch/master?svg=true)](https://ci.appveyor.com/project/sckott/ritis/branch/master)
[![codecov](https://codecov.io/gh/ropensci/ritis/branch/master/graph/badge.svg)](https://codecov.io/gh/ropensci/ritis)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/ritis)](https://github.com/metacran/cranlogs.app)
[![cran version](https://www.r-pkg.org/badges/version/ritis)](https://cran.r-project.org/package=ritis)

An interface to the Integrated Taxonomic Information System (ITIS)

* [ITIS API Docs](https://www.itis.gov/ws_description.html)
* [Solr service](https://www.itis.gov/solr_documentation.html)
* [taxize book](https://taxize.dev/)

## Package API

 - `accepted_names`
 - `any_match_count`
 - `comment_detail`
 - `common_names`
 - `core_metadata`
 - `coverage`
 - `credibility_rating`
 - `credibility_ratings`
 - `currency`
 - `date_data`
 - `description`
 - `experts`
 - `full_record`
 - `geographic_divisions`
 - `geographic_values`
 - `global_species_completeness`
 - `hierarchy_down`
 - `hierarchy_full`
 - `hierarchy_up`
 - `itis_facet`
 - `itis_group`
 - `itis_highlight`
 - `itis_search`
 - `jurisdiction_origin_values`
 - `jurisdiction_values`
 - `jurisdictional_origin`
 - `kingdom_name`
 - `kingdom_names`
 - `last_change_date`
 - `lsid2tsn`
 - `other_sources`
 - `parent_tsn`
 - `publications`
 - `rank_name`
 - `rank_names`
 - `record`
 - `review_year`
 - `scientific_name`
 - `search_any_match_paged`
 - `search_anymatch`
 - `search_common`
 - `search_scientific`
 - `synonym_names`
 - `taxon_authorship`
 - `terms`
 - `tsn_by_vernacular_language`
 - `tsn2lsid`
 - `unacceptability_reason`
 - `usage`
 - `vernacular_languages`


## Installation

Stable, CRAN version


```r
install.packages("ritis")
```

Dev version


```r
devtools::install_github("ropensci/ritis")
```


```r
library("ritis")
```

## Solr service

matches only monomials


```r
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/")
#> # A tibble: 10 x 20
#>    tsn   nameWInd nameWOInd unit1 usage credibilityRati… completenessRat…
#>    <chr> <chr>    <chr>     <chr> <chr> <chr>            <chr>           
#>  1 1348… Phthiri… Phthirii… Phth… valid No review; untr… unknown         
#>  2 1348… Phthiria Phthiria  Phth… valid No review; untr… unknown         
#>  3 1348… Poecilo… Poecilog… Poec… valid No review; untr… unknown         
#>  4 1348… Neacreo… Neacreot… Neac… valid No review; untr… unknown         
#>  5 1348… Geronin… Geroninae Gero… valid No review; untr… unknown         
#>  6 1348… Geron    Geron     Geron valid No review; untr… unknown         
#>  7 1349… Cythere… Cytherei… Cyth… valid No review; untr… unknown         
#>  8 1349… Pantarb… Pantarbes Pant… valid No review; untr… unknown         
#>  9 1349… Tomomyz… Tomomyzi… Tomo… valid No review; untr… unknown         
#> 10 1349… Amphico… Amphicos… Amph… valid No review; untr… unknown         
#> # … with 13 more variables: currencyRating <chr>, kingdom <chr>,
#> #   parentTSN <chr>, rankID <chr>, rank <chr>, hierarchySoFar <chr>,
#> #   hierarchySoFarWRanks <chr>, hierarchyTSN <chr>, otherSource <chr>,
#> #   createDate <chr>, updateDate <chr>, hierarchicalSort <chr>,
#> #   `_version_` <dbl>
```

matches only binomials


```r
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")
#> # A tibble: 10 x 24
#>    tsn   nameWInd nameWOInd unit1 unit2 usage unacceptReason
#>    <chr> <chr>    <chr>     <chr> <chr> <chr> <chr>         
#>  1 1347… Rhabdop… Rhabdops… Rhab… mus   inva… junior synonym
#>  2 1347… Apolysi… Apolysis… Apol… neut… valid <NA>          
#>  3 1347… Pseudog… Pseudoge… Pseu… neut… inva… junior synonym
#>  4 1347… Apolysi… Apolysis… Apol… nigr… valid <NA>          
#>  5 1347… Oligodr… Oligodra… Olig… nigr… inva… junior synonym
#>  6 1347… Apolysi… Apolysis… Apol… obsc… valid <NA>          
#>  7 1347… Pseudog… Pseudoge… Pseu… obsc… inva… junior synonym
#>  8 1347… Apolysi… Apolysis… Apol… palp… valid <NA>          
#>  9 1347… Oligodr… Oligodra… Olig… palp… inva… junior synonym
#> 10 1347… Apolysi… Apolysis… Apol… pann… valid <NA>          
#> # … with 17 more variables: credibilityRating <chr>, taxonAuthor <chr>,
#> #   kingdom <chr>, rankID <chr>, rank <chr>, hierarchySoFar <chr>,
#> #   hierarchySoFarWRanks <chr>, hierarchyTSN <chr>, synonyms <chr>,
#> #   synonymTSNs <chr>, otherSource <chr>, acceptedTSN <chr>,
#> #   createDate <chr>, updateDate <chr>, `_version_` <dbl>,
#> #   parentTSN <chr>, hierarchicalSort <chr>
```

## REST API

Get accepted names for a TSN


```r
accepted_names(tsn = 504239)
#> # A tibble: 1 x 3
#>   acceptedName        acceptedTsn author    
#>   <chr>               <chr>       <chr>     
#> 1 Dasiphora fruticosa 836659      (L.) Rydb.
```

Get common names for a TSN


```r
common_names(tsn = 183833)
#> # A tibble: 3 x 3
#>   commonName          language tsn   
#>   <chr>               <chr>    <chr> 
#> 1 African hunting dog English  183833
#> 2 African Wild Dog    English  183833
#> 3 Painted Hunting Dog English  183833
```

Full hierarchy for a TSN


```r
hierarchy_full(tsn = 37906)
#> # A tibble: 60 x 5
#>    parentname      parenttsn rankname      taxonname       tsn   
#>    <chr>           <chr>     <chr>         <chr>           <chr> 
#>  1 ""              ""        Kingdom       Plantae         202422
#>  2 Plantae         202422    Subkingdom    Viridiplantae   954898
#>  3 Viridiplantae   954898    Infrakingdom  Streptophyta    846494
#>  4 Streptophyta    846494    Superdivision Embryophyta     954900
#>  5 Embryophyta     954900    Division      Tracheophyta    846496
#>  6 Tracheophyta    846496    Subdivision   Spermatophytina 846504
#>  7 Spermatophytina 846504    Class         Magnoliopsida   18063 
#>  8 Magnoliopsida   18063     Superorder    Asteranae       846535
#>  9 Asteranae       846535    Order         Asterales       35419 
#> 10 Asterales       35419     Family        Asteraceae      35420 
#> # … with 50 more rows
```

## Meta

* Please [report any issues or bugs](https://github.com/ropensci/ritis/issues).
* License: MIT
* Get citation information for `ritis` in R doing `citation(package = 'ritis')`
* Please note that this project is released with a [Contributor Code of Conduct][coc].
By participating in this project you agree to abide by its terms.

[![ropensci_footer](https://ropensci.org/public_images/github_footer.png)](https://ropensci.org)

[coc]: https://github.com/ropensci/ritis/blob/master/CODE_OF_CONDUCT.md

#' Returns the kingdom and rank information for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame, with rank name and other info
#' @examples \dontrun{
#' rank_name(tsn = 202385)
#' }
rank_name <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getTaxonomicRankNameFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)
  res <- tc(pick_cols(x, c("kingdomId","kingdomName","rankId","rankName","tsn")))
  tibble::as_tibble(
    if (length(names(res)) == 1) NULL else res
  )
}

#' Get core metadata from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @examples \dontrun{
#' # coverage and currrency data
#' core_metadata(tsn=28727)
#' core_metadata(tsn=28727, wt = "xml")
#' # no coverage or currrency data
#' core_metadata(183671)
#' core_metadata(183671, wt = "xml")
#' }
core_metadata <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCoreMetadataFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tibble::as_tibble(parse_raw(out)), "class")
}

context("core_metadata")

skip_on_cran()

test_that("core_metadata basic functionality works", {
  vcr::use_cassette("core_metadata", {
    aa <- core_metadata(tsn = 183833)
    bb <- core_metadata(tsn = 183671)
  })
  
  # coverage and currrency data
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 1)
  expect_is(aa$taxonUsageRating, "character")
  expect_equal(aa$taxonUsageRating, "valid")

  # no coverage or currrency data
  expect_is(bb, "data.frame")
  expect_equal(NROW(bb), 1)
  expect_is(bb$taxonUsageRating, "character")
  expect_equal(bb$taxonUsageRating, "not accepted")
})

test_that("core_metadata - xml works", {
  vcr::use_cassette("core_metadata-xml", {
    aa <- core_metadata(183833, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("core_metadata - raw JSON works", {
  vcr::use_cassette("core_metadata-json", {
    aa <- core_metadata(183833, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("core_metadata fails well", {
  expect_error(core_metadata(), "\"tsn\" is missing")

  expect_error(core_metadata("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("core_metadata-errors", {
    expect_error(core_metadata("asdfadf"))
  })
})

#' Get full record from TSN or lsid
#'
#' @export
#' @param lsid lsid for a taxonomic group (character)
#' @inheritParams comment_detail
#' @examples \dontrun{
#' # from tsn
#' full_record(tsn = 50423)
#' full_record(tsn = 202385)
#' full_record(tsn = 183833)
#'
#' full_record(tsn = 183833, wt = "xml")
#' full_record(tsn = 183833, raw = TRUE)
#'
#' # from lsid
#' full_record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
#' full_record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
#' }
full_record <- function(tsn = NULL, lsid = NULL, wt = "json", raw = FALSE,
                        ...) {

  if (!xor(is.null(tsn), is.null(lsid))) {
    stop("Pass only one of `tsn` or `lsid`", call. = FALSE)
  }
  if (!is.null(tsn)) {
    verb <- "getFullRecordFromTSN"
    args <- list(tsn = tsn)
  } else {
    verb <- "getFullRecordFromLSID"
    args <- list(lsid = lsid)
  }
  out <- itis_GET(verb, args, wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tc(parse_raw(out)), "class")
}

context("full_record")

skip_on_cran()

test_that("full_record tsn", {
  vcr::use_cassette("full_record_tsn", {
    aa <- full_record(tsn = 202385)
  })
  
  expect_is(aa, "list")
  expect_is(aa$publicationList, "list")
  expect_is(aa$coreMetadata, "list")
})

test_that("full_record lsid", {
  vcr::use_cassette("full_record_lsid", {
    bb <- full_record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
  })
  
  expect_is(bb, "list")
  expect_is(bb$publicationList, "list")
  expect_is(bb$coreMetadata, "list")
})

test_that("full_record fails well", {
  expect_error(full_record(4, 5), "only one of")

  vcr::use_cassette("full_record-errors", {
    expect_error(full_record("asdfadf"))
  })
})

#' Get geographic divisions from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @examples \dontrun{
#' geographic_divisions(tsn = 180543)
#'
#' geographic_divisions(tsn = 180543, wt = "xml")
#'
#' geographic_divisions(tsn = 180543, wt = "json", raw = TRUE)
#' }
geographic_divisions <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getGeographicDivisionsFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- parse_raw(out)
  if (inherits(res$geoDivisions, "logical") || is.null(res$geoDivisions)) {
    tibble::tibble()
  } else {
    dr_op(tibble::as_tibble(res$geoDivisions), "class")
  }
}

context("geographic_divisions")

skip_on_cran()

test_that("geographic_divisions", {
  vcr::use_cassette("geographic_divisions", {
    aa <- geographic_divisions(tsn = 180543)
  })
  
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 3)
  for (i in aa) expect_is(i, "character")
})

test_that("geographic_divisions - xml works", {
  vcr::use_cassette("geographic_divisions-xml", {
    aa <- geographic_divisions(180544, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("geographic_divisions - raw JSON works", {
  vcr::use_cassette("geographic_divisions-json", {
    aa <- geographic_divisions(180544, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("geographic_divisions fails well", {
  expect_error(geographic_divisions(), "\"tsn\" is missing")

  expect_error(geographic_divisions("asdfafasffd", wt = "ffa"),
    "'wt' must be one of")

  vcr::use_cassette("geographic_divisions-errors", {
    expect_error(geographic_divisions("asdfadf"))
  })
})

#' Returns a list of the synonyms (if any) for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame
#' @examples \dontrun{
#' synonym_names(tsn=183671) # tsn not accepted
#' synonym_names(tsn=526852) # tsn accepted
#' }
synonym_names <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getSynonymNamesFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  tmp <- parse_raw(out)$synonyms
  if (inherits(tmp, "logical") || is.null(tmp)) {
    tibble::tibble()
  } else {
    dr_op(tibble::as_tibble(tmp), "class")
  }
}

tc <- function(l) Filter(Negate(is.null), l)

argsnull <- function(x) {
  if (length(x) == 0) {
    NULL
  } else {
    x
  }
}

nmslwr <- function(x) {
  stats::setNames(x, tolower(names(x)))
}

itbase <- function() 'https://www.itis.gov/ITISWebService/services/ITISService/'
itjson <- function() 'https://www.itis.gov/ITISWebService/jsonservice/'
itis_solr_url <- function() "https://services.itis.gov"
iturl <- function(x) {
  if (!tolower(x) %in% c('json', 'xml')) {
    stop("'wt' must be one of 'json' or 'xml'", call. = FALSE)
  }
  switch(
    x,
    json = itjson(),
    xml = itbase()
  )
}

`%-%` <- function(x, y) if (length(x) == 0 || nchar(x) == 0 || is.null(x)) y else x

dr_op <- function(x, y) UseMethod("dr_op")
dr_op.default <- function(x, y) return(NULL)
dr_op.data.frame <- function(x, y) x[, !tolower(names(x)) %in% tolower(y)]
dr_op.list <- function(x, y) x[!tolower(names(x)) %in% tolower(y)]

itis_GET <- function(endpt, args, wt, ...){
  args <- argsnull(args)
  cli <- crul::HttpClient$new(
    url = paste0(iturl(wt), endpt),
    opts = list(...)
  )
  tt <- cli$get(query = args)
  tt$raise_for_status()

  # sort out encoding - if not found, parse differently
  encoding <- NULL
  if (!is.null(tt$response_headers$`content-type`)) {
    encoding <- strsplit(
      strsplit(tt$response_headers$`content-type`, ";")[[1]][2],
      "="
    )[[1]][2]
  }
  if (is.null(encoding) || !nzchar(encoding)) {
    readBin(tt$content, character())
  } else {
    tt$parse(encoding)
  }
}

parse_raw <- function(x) {
  if ((inherits(x, "character") && !nzchar(x)) || is.na(x)) {
    return(tibble::as_tibble())
  }
  jsonlite::fromJSON(x, flatten = TRUE)
}

pick_cols <- function(x, nms) {
  UseMethod("pick_cols")
}

pick_cols.default <- function(x, nms) {
  return(NULL)
}

pick_cols.data.frame <- function(x, nms) {
  if (NROW(x) > 0) {
    names(x) <- tolower(names(x))
    x[, names(x) %in% tolower(nms)]
  } else {
    NULL
  }
}

pick_cols.list <- function(x, nms) {
  if (NROW(x) > 0) {
    names(x) <- tolower(names(x))
    x[names(x) %in% tolower(nms)]
  } else {
    NULL
  }
}

make_itis_conn <- function(proxy) {
  solrium::SolrClient$new(host = "services.itis.gov",
    scheme = "https", port = NULL, errors = "complete",
    proxy = proxy)
}

#' Get accepted names from tsn
#'
#' @export
#' @param wt (character) One of "json" or "xml". Required.
#' @param raw (logical) Return raw JSON or XML as character string. Required.
#' Default: `FALSE`
#' @param ... curl options passed on to [crul::HttpClient]
#' @template tsn
#' @return Zero row data.frame if the name is accepted, otherwise a data.frame
#' with information on the currently accepted name
#' @examples \dontrun{
#' # TSN accepted - good name, empty data.frame returned
#' accepted_names(tsn = 208527)
#'
#' # TSN not accepted - input TSN is old name, non-empty data.frame returned
#' accepted_names(tsn = 504239)
#'
#' # raw json
#' accepted_names(tsn = 208527, raw = TRUE)
#' }
accepted_names <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getAcceptedNamesFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  tmp <- parse_raw(out)
  if (all(is.na(tmp$acceptedNames))) {
    tibble::tibble()
  } else {
    dr_op(tibble::as_tibble(tmp$acceptedNames), "class")
  }
}

context("accepted_names")

skip_on_cran()

test_that("accepted_names basic functionality works", {
  vcr::use_cassette("accepted_names", {
    aa <- accepted_names(tsn = 208527)
    bb <- accepted_names(tsn = 504239)
  })
  
  # TSN accepted - good name, empty data.frame returned
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 0)

  # TSN not accepted - input TSN is old name, non-empty data.frame returned
  expect_is(bb, "data.frame")
  expect_equal(NROW(bb), 1)
  expect_equal(bb$acceptedName, "Dasiphora fruticosa")
})

test_that("accepted_names - xml works", {
  vcr::use_cassette("accepted_names-xml", {
    aa <- accepted_names(504239, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("accepted_names - raw JSON works", {
  vcr::use_cassette("accepted_names-json", {
    aa <- accepted_names(504239, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("accepted_names fails well", {
  expect_error(accepted_names(), "\"tsn\" is missing")

  expect_error(accepted_names("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("accepted_names-errors", {
    expect_error(accepted_names("asdfadf"))
  })
})

#' ITIS Solr group search
#'
#' @export
#' @param ... Arguments passed on to the `params` parameter of
#' the [solrium::solr_group()] function. See [solr_fields] for possible
#' parameters, and examples below
#' @inheritParams itis_search
#' @examples \dontrun{
#' x <- itis_group(q = "nameWOInd:/[A-Za-z0-9]*[%20]{0,0}*/",
#'    group.field = 'rank', group.limit = 3)
#' head(x)
#' }
itis_group <- function(..., proxy = NULL, callopts=list()) {
  if (!is.null(proxy)) conn_dc <- make_itis_conn(proxy)
  args <- list(...)
	if (!is.null(args$fl)) args$fl <- paste(args$fl, collapse = ",")
  conn_itis$group(params = args, callopts = callopts)
}

context("itis_group")

test_that("itis_group basic functionality works", {
  vcr::use_cassette("itis_group", {
    aa <- sm(itis_group(q = "nameWOInd:/[A-Za-z0-9]*[%20]{0,0}*/",
      group.field = 'rank', group.limit = 3))
  })

  expect_is(aa, "data.frame")
  expect_true(any(grepl("group", names(aa))))
})

test_that("itis_group fails well", {
  skip_on_cran()

  vcr::use_cassette("itis_group-fails-well", {
    expect_error(sm(itis_group()))
  })

  expect_error(sm(itis_group(wt = "asdfaddf")))
})

#' Provides a list of all the unique rank names contained in the database and
#'  their kingdom and rank ID values.
#'
#' @export
#' @inheritParams accepted_names
#' @return a data.frame, with columns:
#' \itemize{
#'  \item kingdomname
#'  \item rankid
#'  \item rankname
#' }
#' @examples \dontrun{
#' rank_names()
#' }
rank_names <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getRankNames", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)$rankNames
  tibble::as_tibble(pick_cols(
    x,
    c("kingdomName", "rankId", "rankName")
  ))
}

#' Get expert information for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @examples \dontrun{
#' experts(tsn = 180544)
#' experts(180544, wt = "xml")
#' experts(180544, raw = TRUE)
#' }
experts <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getExpertsFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tibble::as_tibble(parse_raw(out)$experts), "class")
}

context("experts")

skip_on_cran()

test_that("experts basic functionality works", {
  vcr::use_cassette("experts", {
    aa <- experts(tsn = 180544)
  })
  
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 2)
  expect_is(aa$comment, "character")
  expect_is(aa$referenceFor, "list")
})

test_that("experts - xml works", {
  vcr::use_cassette("experts-xml", {
    aa <- experts(180544, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("experts - raw JSON works", {
  vcr::use_cassette("experts-json", {
    aa <- experts(180544, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("experts fails well", {
  expect_error(experts(), "\"tsn\" is missing")

  expect_error(experts("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("experts-errors", {
    expect_error(experts("asdfadf"))
  })
})

#' Get date data from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @examples \dontrun{
#' date_data(tsn = 180543)
#' date_data(180543, wt = "xml")
#' date_data(180543, wt = "json", raw = TRUE)
#' }
date_data <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getDateDataFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- parse_raw(out)
  if (is.null(res$initialTimeStamp)) res$initialTimeStamp <- ""
  if (is.null(res$updateDate)) res$updateDate <- ""
  dr_op(tibble::as_tibble(res), "class")
}

context("date_data")

skip_on_cran()

test_that("date_data basic functionality works", {
  vcr::use_cassette("date_data", {
    aa <- date_data(tsn = 28727)
  })
  
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 1)
  expect_is(aa$updateDate, "character")
})

test_that("date_data - xml works", {
  vcr::use_cassette("date_data-xml", {
    aa <- date_data(28727, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("date_data - raw JSON works", {
  vcr::use_cassette("date_data-json", {
    aa <- date_data(28727, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("date_data fails well", {
  expect_error(date_data(), "\"tsn\" is missing")

  expect_error(date_data("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("date_data-errors", {
    expect_error(date_data("asdfadf"))
  })
})

#' ITIS Solr highlight
#'
#' @export
#' @param ... Arguments passed on to the `params` parameter of
#' the [solrium::solr_highlight()] function. See [solr_fields] for possible
#' parameters, and examples below
#' @inheritParams itis_search
#' @examples \dontrun{
#' itis_highlight(q = "rank:Species", hl.fl = 'rank', rows=10)
#' }
itis_highlight <- function(..., proxy = NULL, callopts=list()) {
  if (!is.null(proxy)) conn_dc <- make_itis_conn(proxy)
  args <- list(...)
	if (!is.null(args$fl)) args$fl <- paste(args$fl, collapse = ",")
  conn_itis$highlight(params = args, callopts = callopts)
}

context("itis_highlight")

test_that("itis_highlight basic functionality works", {
  vcr::use_cassette("itis_highlight", {
    aa <- sm(itis_highlight(q = "rank:Species", hl.fl = 'rank', rows = 10))
  })

  expect_is(aa, "tbl_df")
  expect_is(aa$rank[1], "character")
  expect_is(aa$names[1], "character")
  expect_named(aa, c("names", "rank"))
})

test_that("itis_highlight fails well", {
  skip_on_cran()

  expect_error(sm(itis_highlight(foo = "bar")),
               "some keys not in acceptable set")

  expect_error(sm(itis_highlight(wt = "asdfaddf")),
    "wt must be one of")
})

#' Get jurisdictional origin from tsn
#'
#' @export
#' @name jurisdiction
#' @inheritParams accepted_names
#' @template tsn
#' @return
#' \itemize{
#'  \item jurisdictional_origin: data.frame
#'  \item jurisdiction_origin_values: data.frame
#'  \item jurisdiction_values: character vector
#' }
#' @details Jurisdiction methods:
#' \itemize{
#'  \item jurisdictional_origin: Get jurisdictional origin from tsn
#'  \item jurisdiction_origin_values: Get jurisdiction origin values
#'  \item jurisdiction_values: Get all possible jurisdiction values
#' }
#' @examples \dontrun{
#' jurisdictional_origin(tsn=180543)
#' jurisdictional_origin(tsn=180543, wt = "xml")
#'
#' jurisdiction_origin_values()
#'
#' jurisdiction_values()
#' }
jurisdictional_origin <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getJurisdictionalOriginFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  z <- parse_raw(out)
  tibble::as_tibble(
    pick_cols(z$jurisdictionalOrigins, c("jurisdictionValue", "origin",
                                         "updateDate"))
  )
}

#' @export
#' @rdname jurisdiction
jurisdiction_origin_values <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getJurisdictionalOriginValues", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  tibble::as_tibble(
    pick_cols(parse_raw(out)$originValues, c("jurisdiction", "origin"))
  )
}

#' @export
#' @rdname jurisdiction
jurisdiction_values <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getJurisdictionValues", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$jurisdictionValues
}

context("jurisdiction functions")

test_that("jurisdictional_origin basic functionality works", {
  vcr::use_cassette("jurisdictional_origin", {
    aa <- jurisdictional_origin(tsn=180543)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$origin, "character")
  expect_gt(NROW(aa), 0)
})

test_that("jurisdiction_origin_values - basic functionality works", {
  vcr::use_cassette("jurisdictional_origin_values", {
    aa <- jurisdiction_origin_values()
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_named(aa, c('jurisdiction', 'origin'))
})

test_that("jurisdiction_values - basic functionality works", {
  vcr::use_cassette("jurisdiction_values", {
    aa <- jurisdiction_values()
  })

  expect_is(aa, "character")
  expect_gt(length(aa), 1)
})

test_that("jurisdiction functions fail well", {
  
  expect_error(jurisdictional_origin(), "\"tsn\" is missing")

  expect_error(jurisdiction_origin_values(wt = "ffa"), "'wt' must be one of")

  expect_error(jurisdiction_values(wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("jurisdictional_origin-fails-well", {
    # lsid's not found lead 404
    expect_error(jurisdictional_origin(tsn = "asdfasdf"),
      "Bad Request \\(HTTP 400\\)", class = "error")
  })
})

#' Get any match count.
#'
#' @export
#' @param x text or taxonomic serial number (TSN) (character or numeric)
#' @inheritParams accepted_names
#' @return An integer containing the number of matches the search will return.
#' @examples \dontrun{
#' any_match_count(x = 202385)
#' any_match_count(x = "dolphin")
#' any_match_count(x = "dolphin", wt = "xml")
#' }
any_match_count <- function(x, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getAnyMatchCount", list(srchKey = x), wt = wt, ...)
  if (raw || wt == "xml") return(out)
  if (!nzchar(out)) 0 else parse_raw(out)$return
}

context("any_match_count")

test_that("any_match_count basic functionality works", {
  vcr::use_cassette("any_match_count", {
    aa <- any_match_count(x = 202385)
    bb <- any_match_count(x = "dolphin")
  })
  
  expect_true(class(aa) %in% c('numeric', 'integer'))
  expect_gt(aa, 0)
  expect_true(class(bb) %in% c('numeric', 'integer'))
  expect_gt(bb, 0)
})

test_that("any_match_count - xml works", {
  vcr::use_cassette("any_match_count-xml", {
    aa <- any_match_count(202385, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("any_match_count - raw JSON works", {
  vcr::use_cassette("any_match_count-json", {
    aa <- any_match_count(202385, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("any_match_count fails well", {
  skip_on_cran()

  expect_error(any_match_count(), "\"x\" is missing")

  expect_error(any_match_count("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("any_match_count-fails-well", {
    # character string query with no results lead to values of "0"
    tmp1 <- any_match_count(x = "asdfadf")
    # numeric query with no results lead to values of "0"
    tmp2 <- any_match_count(x = 343423432423424)
  })
  
  expect_equal(tmp1, 0)
  expect_equal(tmp2, 0)
})

## ---- echo = FALSE, warning = FALSE, message = FALSE---------------------
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse  =  TRUE,
  comment  =  "#>",
  warning = FALSE,
  message = FALSE,
  purl  =  NOT_CRAN,
  eval  =  NOT_CRAN
)

#' Returns the unacceptability reason, if any, for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @examples \dontrun{
#' unacceptability_reason(tsn = 183671)
#' }
unacceptability_reason <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getUnacceptabilityReasonFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)
  if (is.null(x$unacceptReason) || inherits(x$unacceptReason, "logical")) {
    tibble::tibble()
  } else {
    dr_op(tibble::as_tibble(x), "class")
  }
}

#' Get all possible geographic values
#'
#' @export
#' @inheritParams accepted_names
#' @return character vector of geographic names
#' @examples \dontrun{
#' geographic_values()
#' geographic_values(wt = "xml")
#' geographic_values(wt = "json", raw = TRUE)
#' }
geographic_values <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getGeographicValues", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$geographicValues
}

context("geographic_values")

skip_on_cran()

test_that("geographic_values", {
  vcr::use_cassette("geographic_values", {
    aa <- geographic_values()
  })
  
  expect_is(aa, "character")
  expect_gt(length(aa), 10)
})

test_that("geographic_values - xml works", {
  vcr::use_cassette("geographic_values-xml", {
    aa <- geographic_values(wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("geographic_values - raw JSON works", {
  vcr::use_cassette("geographic_values-json", {
    aa <- geographic_values(raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("geographic_values fails well", {
  expect_error(geographic_values(wt = "ffa"),
    "'wt' must be one of")
})

#' Get kingdom names from tsn
#'
#' @export
#' @name kingdoms
#' @inheritParams accepted_names
#' @template tsn
#' @details
#' \itemize{
#'  \item kingdom_name: Get kingdom name for a TSN
#'  \item kingdom_names: Get all possible kingdom names
#' }
#' @examples \dontrun{
#' kingdom_name(202385)
#' kingdom_name(202385, wt = "xml")
#' kingdom_names()
#' }
kingdom_name <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getKingdomNameFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- tc(pick_cols(parse_raw(out), c('kingdomid', 'kingdomname', 'tsn')))
  tibble::as_tibble(
    if (length(names(res)) != 1) res else NULL
  )
}

#' @export
#' @rdname kingdoms
kingdom_names <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getKingdomNames", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  pick_cols(tibble::as_tibble(parse_raw(out)$kingdomNames), c('kingdomid', 'kingdomname', 'tsn'))
}

context("kingdom functions")

test_that("kingdom_name basic functionality works", {
  vcr::use_cassette("kingdom_name", {
    aa <- kingdom_name(202385)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_is(aa$kingdomname, "character")
  expect_equal(aa$kingdomname, "Animalia")
  expect_gt(NROW(aa), 0)
})

test_that("kingdom_names basic functionality works", {
  vcr::use_cassette("kingdom_names", {
    aa <- kingdom_names()
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_gt(NROW(aa), 0)
  expect_true(any(grepl("Fungi", aa$kingdomname)))
})

test_that("kingdom functions fail well", {
  expect_error(kingdom_name(), "\"tsn\" is missing")

  expect_error(kingdom_names(wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("kingdom_name-fails-well", {
    # tsn's not found lead to 404
    expect_error(kingdom_name(tsn = "Asdfasdfa"),
      "Bad Request \\(HTTP 400\\)", class = "error")
  })
})

#' Returns the parent TSN for the entered TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame
#' @examples \dontrun{
#' parent_tsn(tsn = 202385)
#' parent_tsn(tsn = 202385, raw = TRUE)
#' parent_tsn(tsn = 202385, wt = "xml")
#' }
parent_tsn <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getParentTSNFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)
  res <- tc(pick_cols(x, c("parentTsn", "tsn")))
  tibble::as_tibble(
    if (length(names(res)) == 1) NULL else res
  )
}

#' Provides the date the ITIS database was last updated
#'
#' @export
#' @inheritParams accepted_names
#' @return character value with a date
#' @examples \dontrun{
#' last_change_date()
#' last_change_date(wt = "xml")
#' }
last_change_date <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getLastChangeDate", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$updateDate
}

#' Get currency from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @return a data.frame
#' @examples \dontrun{
#' # currency data
#' currency(tsn=28727)
#' currency(tsn=28727, wt = "xml")
#' # no currency dat
#' currency(526852)
#' currency(526852, raw = TRUE)
#' }
currency <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCurrencyFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- parse_raw(out)
  if (is.null(res$taxonCurrency)) res$taxonCurrency <- ""
  dr_op(tibble::as_tibble(res), "class")
}

#' Gets the unique LSID for the TSN, or an empty result if there is no match.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a character string, an LSID, or \code{NULL} if nothing found
#' @examples \dontrun{
#' tsn2lsid(tsn = 155166)
#' tsn2lsid(tsn = 333333333)
#' tsn2lsid(155166, raw = TRUE)
#' tsn2lsid(155166, wt = "xml")
#' }
tsn2lsid <- function(tsn, wt = "json", raw = FALSE, ...) {
  x <- itis_GET("getLSIDFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(x)
  parse_raw(x)$return
}

#' ITIS Solr Methods
#'
#' ITIS provides access to their data via their Solr service described at
#' <https://www.itis.gov/solr_documentation.html>.  This is a powerful
#' interace to ITIS data as you have access to a very flexible query
#' interface.
#' 
#' See [solr_fields] and <https://www.itis.gov/solr_documentation.html> 
#' for guidance on available fields.
#'
#' @section Functions:
#'
#'  - [itis_search()] - Search
#'  - [itis_group()] - Group
#'  - [itis_highlight()] - Highlight
#'  - [itis_facet()] - Facet
#'
#' @name solr
NULL

#' Get credibility rating from tsn
#'
#' @export
#' @name credibility
#' @template tsn
#' @inheritParams accepted_names
#' @return a data.frame
#' @details methods:
#' \itemize{
#'  \item credibility_rating: Get credibility rating for a tsn
#'  \item credibility_ratings: Get possible credibility ratings
#' }
#' @examples \dontrun{
#' credibility_rating(tsn = 526852)
#' credibility_rating(526852, wt = "xml")
#' credibility_rating(526852, raw = TRUE)
#'
#' credibility_ratings()
#' credibility_ratings(wt = "xml")
#' credibility_ratings(raw = TRUE)
#' }
credibility_rating <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCredibilityRatingFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- parse_raw(out)
  if (is.null(res$credRating)) res$credRating <- ""
  dr_op(tibble::as_tibble(res), "class")
}

#' @export
#' @rdname credibility
credibility_ratings <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCredibilityRatings", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tibble::as_tibble(parse_raw(out)), "class")
}

skip_on_cran()


context("credibility_rating")

test_that("credibility_rating basic functionality works", {
  vcr::use_cassette("credibility_rating", {
    aa <- credibility_rating(tsn = 526852)
  })
  
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 1)
  expect_is(aa$credRating, "character")
  expect_equal(aa$credRating, "TWG standards met")
})

test_that("credibility_rating - xml works", {
  vcr::use_cassette("credibility_rating-xml", {
    aa <- credibility_rating(28727, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("credibility_rating - raw JSON works", {
  vcr::use_cassette("credibility_rating-json", {
    aa <- credibility_rating(28727, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("credibility_rating fails well", {
  expect_error(credibility_rating(), "\"tsn\" is missing")

  expect_error(credibility_rating("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("credibility_rating-errors", {
    expect_error(credibility_rating("asdfadf"))
  })
})



context("credibility_ratings")

test_that("credibility_ratings basic functionality works", {
  vcr::use_cassette("credibility_ratings", {
    aa <- credibility_ratings()
  })
  
  expect_is(aa, "data.frame")
  expect_is(aa$credibilityValues, "character")
})

#' Returns the scientific name for the TSN. Also returns the component parts
#'    (names and indicators) of the scientific name.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame
#' @examples \dontrun{
#' scientific_name(tsn = 531894)
#' }
scientific_name <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getScientificNameFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- tc(parse_raw(out))
  tibble::as_tibble(pick_cols(
    data.frame(x, stringsAsFactors = FALSE),
    c("combinedName","unitInd1","unitInd3","unitName1","unitName2",
      "unitName3","tsn")
  ))
}

#' Returns a list of the other sources used for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @examples \dontrun{
#' # results
#' other_sources(tsn=182662)
#' # no results
#' other_sources(tsn=2085272) 
#' # get xml
#' other_sources(tsn=182662, wt = "xml")
#' }
other_sources <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getOtherSourcesFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)$otherSources
  if (inherits(x, "logical") || is.null(x)) {
    tibble::tibble()
  } else {
    x <- cbind(dr_op(x, "referencefor"), bindlist(x$referenceFor))
    tibble::as_tibble(pick_cols(
      x,
      c("acquisitiondate","name","referredtsn","source", "sourcetype","updatedate","version")
    ))
  }
}

bindlist <- function(x) {
  (data.table::setDF(data.table::rbindlist(x, use.names = TRUE, fill = TRUE)))
}

#' Search by scientific name
#'
#' @export
#' @inheritParams accepted_names
#' @inheritParams any_match_count
#' @return a data.frame
#' @seealso \code{\link{search_common}}
#' @examples \dontrun{
#' search_scientific("Tardigrada")
#' search_scientific("Quercus douglasii")
#' }
search_scientific <- function(x, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("searchByScientificName", list(srchKey = x), wt, ...)
  if (raw || wt == "xml") return(out)
  tibble::as_tibble(
    dr_op(parse_raw(out)$scientificNames, "class")
  )
}

context("search_scientific")

test_that("search_scientific basic functionality works", {
  vcr::use_cassette("search_scientific", {
    aa <- search_scientific("Tardigrada")

    expect_is(aa, "data.frame")
    expect_is(aa, "tbl_df")

    expect_is(aa$tsn, "character")
    expect_gt(NROW(aa), 0)
  })
})

test_that("search_scientific - xml works", {
  vcr::use_cassette("search_scientific-xml", {
    aa <- search_scientific("Tardigrada", wt = "xml")

    expect_is(aa, "character")
    expect_true(grepl("xmlns", aa))
  })
})

test_that("search_scientific - raw JSON works", {
  vcr::use_cassette("search_scientific-json", {
    aa <- search_scientific("Tardigrada", raw = TRUE)

    expect_is(aa, "character")
    expect_false(grepl("xmlns", aa))
  })
})

test_that("search_scientific fails well", {
  expect_error(search_scientific(), "\"x\" is missing")
  expect_error(search_scientific("asdfadf", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("search_scientific-fail-well", {
    # query with no results lead to 0 row data.frame's
    tmp <- search_scientific(x = "asdfadf", wt = "json")
    expect_is(tmp, "tbl_df")
    expect_equal(NROW(tmp), 0)
  })
})

#' Get tsn by vernacular language
#'
#' @export
#' @inheritParams accepted_names
#' @param language A string containing the language. This is a language string,
#'    not the international language code (character)
#' @return a data.frame
#' @examples \dontrun{
#' tsn_by_vernacular_language(language = "french")
#' }
tsn_by_vernacular_language <- function(language, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getTsnByVernacularLanguage", list(language = language), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)
  if (is.null(x$vernacularTsns) || inherits(x$vernacularTsns, "logical")) {
    tibble::tibble()
  } else {
    dr_op(tibble::as_tibble(x$vernacularTsns), "class")
  }
}

#' Search for tsn by common name
#'
#' @export
#' @param x text or taxonomic serial number (TSN) (character or numeric)
#' @param from (character) One of "all", "begin", or "end". See Details.
#' @inheritParams accepted_names
#' @details The `from` parameter:
#' \itemize{
#'  \item all - Search against the `searchByCommonName` API route, which
#'  searches entire name string
#'  \item begin - Search against the `searchByCommonNameBeginsWith` API
#'  route, which searches for a match at the beginning of a name string
#'  \item end - Search against the `searchByCommonNameEndsWith` API route,
#'  which searches for a match at the end of a name string
#' }
#' @return a data.frame
#' @seealso [search_scientific()]
#' @examples \dontrun{
#' search_common("american bullfrog")
#' search_common("ferret-badger")
#' search_common("polar bear")
#'
#' # comparison: all, begin, end
#' search_common("inch")
#' search_common("inch", from = "begin")
#' search_common("inch", from = "end")
#'
#' # end
#' search_common("snake", from = "end")
#' }
search_common <- function(x, from = "all", wt = "json", raw = FALSE,
                          ...) {
  verb <- switch(from,
                 all = "searchByCommonName",
                 begin = "searchByCommonNameBeginsWith",
                 end = "searchByCommonNameEndsWith")
  out <- itis_GET(endpt = verb, args = list(srchKey = x), wt, ...)
  if (raw || wt == "xml") return(out)
  tibble::as_tibble(dr_op(parse_raw(out)$commonNames, "class"))
}

context("search_common")

test_that("search_common basic functionality works", {
  vcr::use_cassette("search_common", {
    aa <- search_common(x = "american bullfrog")
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_gt(NROW(aa), 0)
})

test_that("search_common - xml works", {
  vcr::use_cassette("search_common-xml", {
    aa <- search_common(x = "american bullfrog", wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("search_common - raw JSON works", {
  vcr::use_cassette("search_common-json", {
    aa <- search_common(x = "american bullfrog", raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("search_common fails well", {
  expect_error(search_common(), "\"x\" is missing")
  expect_error(search_common("asdfadf", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("search_common-fails-well", {
    # query with no results lead to 0 row data.frame's
    tmp <- search_common(x = "asdfadf")
    expect_is(tmp, "tbl_df")
    expect_equal(NROW(tmp), 0)
  })
})

#' Search for any match
#'
#' @export
#' @inheritParams accepted_names
#' @inheritParams any_match_count
#' @return a data.frame
#' @seealso \code{\link{search_any_match_paged}}
#' @examples \dontrun{
#' search_anymatch(x = 202385)
#' search_anymatch(x = "dolphin")
#' # no results
#' search_anymatch(x = "Pisces")
#' }
search_anymatch <- function(x, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("searchForAnyMatch", list(srchKey = x), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)$anyMatchList
  tmp <- dr_op(bindlist(x$commonNameList.commonNames), "class")
  if (NROW(tmp) == 0) return(tibble::tibble())
  names(tmp) <- paste0("common_", names(tmp))
  x <- suppressWarnings(
    cbind(
      dr_op(x, c("commonNameList.commonNames", "commonNameList.class",
                 "commonNameList.tsn", "class")),
      tmp
    )
  )
  tibble::as_tibble(x)
}

#' Search for any matched page
#'
#' @export
#' @inheritParams accepted_names
#' @inheritParams any_match_count
#' @return a data.frame
#' @param pagesize An integer containing the page size (numeric)
#' @param pagenum An integer containing the page number (numeric)
#' @param ascend A boolean containing true for ascending sort order or false
#'    for descending (logical)
#' @return a data.frame
#' @seealso \code{\link{search_anymatch}}
#' @examples \dontrun{
#' search_any_match_paged(x=202385, pagesize=100, pagenum=1, ascend=FALSE)
#' search_any_match_paged(x="Zy", pagesize=100, pagenum=1, ascend=FALSE)
#' }
search_any_match_paged <- function(x, pagesize = NULL, pagenum = NULL, ascend = NULL, wt = "json", raw = FALSE, ...) {
  args <- tc(list(srchKey=x, pageSize=pagesize, pageNum=pagenum, ascend=ascend))
  out <- itis_GET("searchForAnyMatchPaged", args, wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)$anyMatchList
  tmp <- dr_op(bindlist(x$commonNameList.commonNames), "class")
  names(tmp) <- paste0("common_", names(tmp))
  x <- suppressWarnings(
    cbind(
      dr_op(x, c("commonNameList.commonNames", "commonNameList.class", "commonNameList.tsn", "class")),
      tmp
    )
  )
  tibble::as_tibble(x)
}

#' ITIS Solr search
#'
#' @export
#' @param ... Arguments passed on to the `params` parameter of
#' the [solrium::solr_search()] function. See [solr_fields] for possible
#' parameters, and examples below
#' @param proxy List of arguments for a proxy connection,
#' including one or more of: url, port, username, password,
#' and auth. See [crul::proxy()] for  help, which is used to
#' construct the proxy connection.
#' @param callopts Curl options passed on to [crul::HttpClient]
#' @references <https://www.itis.gov/solr_documentation.html>
#' @details
#' The syntax for this function can be a bit hard to grasp. See
#' https://itis.gov/solr_examples.html for help on generating the
#' syntax ITIS wants for specific searches.
#' @examples \dontrun{
#' itis_search(q = "tsn:182662")
#' 
#' # get all orders within class Aves (birds)
#' z <- itis_search(q = "rank:Class AND nameWOInd:Aves")
#' hierarchy_down(z$tsn)
#' 
#' # get taxa "downstream" from a target taxon
#' ## taxize and taxizedb packages have downstream() fxns, but
#' ## you can do a similar thing here by iteratively drilling down
#' ## the taxonomic hierarchy
#' ## here, we get families within Aves
#' library(data.table)
#' aves <- itis_search(q = "rank:Class AND nameWOInd:Aves")
#' aves_orders <- hierarchy_down(aves$tsn)
#' aves_families <- lapply(aves_orders$tsn, hierarchy_down)
#' rbindlist(aves_families)
#'
#' # the tila operator
#' itis_search(q = "nameWOInd:Liquidamber\\ styraciflua~0.4")
#' 
#' # matches only monomials
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/")
#' 
#' # matches only binomials
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")
#' 
#' # matches only trinomials
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")
#' 
#' # matches binomials or trinomials
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*[ ]{0,1}[A-Za-z0-9]*/")
#'
#' itis_search(q = "nameWOInd:Poa\\ annua")
#'
#' # pagination
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/", rows = 2)
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/", rows = 200)
#'
#' # select fields to return
#' itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/",
#'    fl = c('nameWInd', 'tsn'))
#' }
itis_search <- function(..., proxy = NULL, callopts=list()) {
  if (!is.null(proxy)) conn_dc <- make_itis_conn(proxy)
  args <- list(...)
	if (!is.null(args$fl)) args$fl <- paste(args$fl, collapse = ",")
  conn_itis$search(params = args, minOptimizedRows = FALSE,
  	callopts = callopts)
}

context("itis_search")

test_that("itis_search basic functionality works", {
  vcr::use_cassette("itis_search", {
    aa <- sm(itis_search(q = "tsn:182662"))
    bb <- sm(itis_search(q = "nameWOInd:Liquidamber\\ styraciflua~0.4"))
    cc <- sm(itis_search(q = "nameWOInd:/[A-Ba-z0-9]*[%20]{0,0}*/"))
  }, match_requests_on = c("method", "uri", "query"))
  
  expect_is(aa, "data.frame")
  expect_is(bb, "data.frame")
  expect_is(cc, "data.frame")

  expect_is(aa$tsn, "character")
  expect_true(grepl("Liquidambar", bb$nameWInd))
  expect_false(all(grepl("[C-D]", substring(cc$nameWOInd, 1, 1))))
})

test_that("itis_search fails well", {
  skip_on_cran()

  vcr::use_cassette("itis_search-fails-well", {
    expect_equal(length(sm(itis_search(foo = "bar"))), 0)
    expect_equal(length(sm(itis_search())), 0)
  })

  expect_error(itis_search(wt = "asdfaddf"),
               "must be one of")
})

#' Gets a record from an LSID
#'
#' @export
#' @inheritParams accepted_names
#' @param lsid lsid for a taxonomic group (character). Required.
#' @details Gets the partial ITIS record for the TSN in the LSID, found by comparing the
#'  TSN in the search key to the TSN field. Returns an empty result set if
#'  there is no match or the TSN is invalid.
#' @return a data.frame
#' @examples \dontrun{
#' record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
#' }
record <- function(lsid, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getRecordFromLSID", list(lsid = lsid), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- tc(parse_raw(out))
  tibble::as_tibble(
    if (length(names(x)) < 2) {
      NULL
    } else {
      pick_cols(
        data.frame(x, stringsAsFactors = FALSE),
        c("authorship","genusPart","infragenericEpithet",
          "infraspecificEpithet","lsid","nameComplete","nomenclaturalCode",
          "rank","rankString","specificEpithet","uninomial","tsn"))
    }
  )
}

context("record")

test_that("record basic functionality works", {
  vcr::use_cassette("record", {
    aa <- record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$lsid, "character")
  expect_gt(NROW(aa), 0)
})

test_that("record - xml works", {
  vcr::use_cassette("record-xml", {
    aa <- record(lsid = "urn:lsid:itis.gov:itis_tsn:180543", wt = "xml")
  })

  expect_is(aa, "character")
  expect_match(aa, "xmlns")
})

test_that("record - raw JSON works", {
  vcr::use_cassette("record-json", {
    aa <- record(lsid = "urn:lsid:itis.gov:itis_tsn:180543", raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("record fail well", {
  expect_error(record(), "\"lsid\" is missing")

  expect_error(record(lsid = "urn:lsid:itis.gov:itis_tsn:180543", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("record-fails-well", {
    # lsid's not found lead to 0 row data.frame's
    tmp <- record(lsid = "asdfasdf")
    expect_is(tmp, "tbl_df")
    expect_equal(NROW(tmp), 0)
  })
})

#' Get hierarchy down from tsn
#'
#' @export
#' @name hierarchy
#' @inheritParams accepted_names
#' @template tsn
#' @details Hierarchy methods:
#' \itemize{
#'  \item hierarchy_down: Get hierarchy down from tsn
#'  \item hierarchy_up: Get hierarchy up from tsn
#'  \item hierarchy_full: Get full hierarchy from tsn
#' }
#' @examples \dontrun{
#' ## Full down (class Mammalia)
#' hierarchy_down(tsn=179913)
#'
#' ## Full up (genus Agoseris)
#' hierarchy_up(tsn=36485)
#'
#' ## Full hierarchy
#' ### genus Liatris
#' hierarchy_full(tsn=37906)
#' ### get raw data back
#' hierarchy_full(tsn=37906, raw = TRUE)
#' ### genus Baetis, get xml back
#' hierarchy_full(100800, wt = "xml")
#' }
hierarchy_down <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getHierarchyDownFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  tibble::as_tibble(
    pick_cols(parse_raw(out)$hierarchyList, c("parentName","parentTsn",
                                              "rankName","taxonName","tsn"))
  )
}

#' @export
#' @rdname hierarchy
hierarchy_up <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getHierarchyUpFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- tc(pick_cols(parse_raw(out), c("parentName","parentTsn","rankName",
                                        "taxonName","tsn")))
  tibble::as_tibble(
    if (length(names(res)) != 1) res else NULL
  )
}

#' @export
#' @rdname hierarchy
hierarchy_full <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getFullHierarchyFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)
  tibble::as_tibble(
    pick_cols(
      if ('hierarchyList' %in% names(x)) x$hierarchyList else x,
      c('parentname', 'parenttsn', 'rankname', 'taxonname', 'tsn')
    )
  )
}

context("hierarchy functions")

test_that("hierarchy_down basic functionality works", {
  vcr::use_cassette("hierarchy_down", {
    aa <- hierarchy_down(tsn = 179913)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_gt(NROW(aa), 0)
})

test_that("hierarchy_up basic functionality works", {
  vcr::use_cassette("hierarchy_up", {
    aa <- hierarchy_up(tsn = 36485)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_gt(NROW(aa), 0)
})

test_that("hierarchy_full basic functionality works", {
  vcr::use_cassette("hierarchy_full", {
    aa <- hierarchy_full(tsn = 37906)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$tsn, "character")
  expect_gt(NROW(aa), 0)
})

test_that("hierarchy functions fail well", {
  expect_error(hierarchy_down(), "\"tsn\" is missing")
  expect_error(hierarchy_up(), "\"tsn\" is missing")
  expect_error(hierarchy_full(), "\"tsn\" is missing")

  vcr::use_cassette("hierarchy-fails-well", {
    # tsn's not found gives 404
    expect_error(hierarchy_down(tsn = "Asdfasdfa"),
      "Bad Request \\(HTTP 400\\)", class = "error")

    expect_error(hierarchy_up(tsn = "Asdfasdfa"),
      "Bad Request \\(HTTP 400\\)", class = "error")

    expect_error(hierarchy_full(tsn = "Asdfasdfa"),
      "Bad Request \\(HTTP 400\\)", class = "error")
  })
})

#' Gets the TSN corresponding to the LSID, or an empty result if there is no match.
#'
#' @export
#' @inheritParams accepted_names
#' @param lsid (character) lsid for a taxonomic group. Required.
#' @examples \dontrun{
#' lsid2tsn(lsid="urn:lsid:itis.gov:itis_tsn:28726")
#' lsid2tsn(lsid="urn:lsid:itis.gov:itis_tsn:28726", wt = "xml")
#' lsid2tsn("urn:lsid:itis.gov:itis_tsn:0")
#' lsid2tsn("urn:lsid:itis.gov:itis_tsn:0", wt = "xml")
#' }
lsid2tsn <- function(lsid, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getTSNFromLSID", list(lsid = lsid), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$return
}

#' Get common names from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @return a data.frame
#' @examples \dontrun{
#' common_names(tsn=183833)
#' common_names(tsn=183833, wt = "xml")
#' }
common_names <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCommonNamesFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tibble::as_tibble(parse_raw(out)$commonNames), "class")
}

context("common_names")

skip_on_cran()

test_that("common_names basic functionality works", {
  vcr::use_cassette("common_names", {
    aa <- common_names(tsn = 183833)
  })
  
  # TSN accepted - good name, empty data.frame returned
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 3)
  for (i in aa) expect_is(i, "character")
})

test_that("common_names - xml works", {
  vcr::use_cassette("common_names-xml", {
    aa <- common_names(183833, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("common_names - raw JSON works", {
  vcr::use_cassette("common_names-json", {
    aa <- common_names(183833, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("common_names fails well", {
  expect_error(common_names(), "\"tsn\" is missing")

  expect_error(common_names("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("common_names-errors", {
    expect_error(common_names("asdfadf"))
  })
})

#' Get coverge from tsn
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @examples \dontrun{
#' # coverage data
#' coverage(tsn=28727)
#' # no coverage data
#' coverage(526852)
#' coverage(526852, wt = "xml")
#' }
coverage <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCoverageFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  res <- parse_raw(out)
  if (is.null(res$taxonCoverage)) res$taxonCoverage <- ""
  dr_op(tibble::as_tibble(res), "class")
}

context("coverage")

skip_on_cran()

test_that("coverage basic functionality works", {
  vcr::use_cassette("coverage", {
    aa <- coverage(tsn = 28727)
    bb <- coverage(tsn = 526852)
  })
  
  # coverage and currrency data
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 1)
  expect_is(aa$taxonCoverage, "character")
  expect_equal(aa$taxonCoverage, "partial")

  # no coverage or currrency data
  expect_is(bb, "data.frame")
  expect_equal(NROW(bb), 1)
  expect_is(bb$taxonCoverage, "character")
  expect_equal(nchar(bb$taxonCoverage), 0)
})

test_that("coverage - xml works", {
  vcr::use_cassette("coverage-xml", {
    aa <- coverage(28727, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("coverage - raw JSON works", {
  vcr::use_cassette("coverage-json", {
    aa <- coverage(28727, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("coverage fails well", {
  expect_error(coverage(), "\"tsn\" is missing")

  expect_error(coverage("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("coverage-errors", {
    expect_error(coverage("asdfadf"))
  })
})

#' Get comment detail from TSN
#'
#' @export
#' @template tsn
#' @inheritParams accepted_names
#' @return A data.frame with results.
#' @examples \dontrun{
#' comment_detail(tsn=180543)
#' comment_detail(tsn=180543, wt = "xml")
#' }
comment_detail <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getCommentDetailFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  dr_op(tibble::as_tibble(parse_raw(out)$comments), "class")
}

context("comment_detail")

skip_on_cran()

test_that("comment_detail basic functionality works", {
  vcr::use_cassette("comment_detail", {
    aa <- comment_detail(tsn = 180543)
  })
  
  # TSN accepted - good name, empty data.frame returned
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 2)
  for (i in aa) expect_is(i, "character")
})

test_that("comment_detail - xml works", {
  vcr::use_cassette("comment_detail-xml", {
    aa <- comment_detail(180543, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("comment_detail - raw JSON works", {
  vcr::use_cassette("comment_detail-json", {
    aa <- comment_detail(180543, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("comment_detail fails well", {
  expect_error(comment_detail(), "\"tsn\" is missing")

  expect_error(comment_detail("asdfafasffd", wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("comment_detail-errors", {
    expect_error(comment_detail("asdfadf"))
  })
})

#' ITIS Solr facet
#'
#' @export
#' @param ... Arguments passed on to the `params` parameter of
#' the [solrium::solr_facet()] function. See [solr_fields] for possible
#' parameters, and examples below
#' @inheritParams itis_search
#' @examples \dontrun{
#' itis_facet(q = "rank:Species", rows = 0, facet.field = "kingdom")$facet_fields
#'
#' x <- itis_facet(q = "hierarchySoFar:*$Aves$* AND rank:Species AND usage:valid",
#'    facet.pivot = "nameWInd,vernacular", facet.limit = -1, facet.mincount = 1,
#'    rows = 0)
#' head(x$facet_pivot$`nameWInd,vernacular`)
#' }
itis_facet <- function(..., proxy = NULL, callopts=list()) {
  if (!is.null(proxy)) conn_dc <- make_itis_conn(proxy)
  args <- list(...)
	if (!is.null(args$fl)) args$fl <- paste(args$fl, collapse = ",")
  conn_itis$facet(params = args, callopts = callopts)
}

context("itis_facet")

test_that("itis_facet basic functionality works", {
  vcr::use_cassette("itis_facet", {
    aa <- sm(itis_facet(q = "rank:Species", rows = 0, facet.field = "kingdom"))
  })

  expect_is(aa, "list")
  expect_is(aa$facet_fields, "list")
  expect_is(aa$facet_fields[[1]], "data.frame")
  expect_null(aa$facet_queries)
  expect_null(aa$facet_pivot)
  expect_null(aa$facet_dates)
  expect_null(aa$facet_ranges)

  expect_named(aa$facet_fields, "kingdom")
  expect_named(aa$facet_fields[[1]], c('term', 'value'))
})

test_that("itis_facet fails well", {
  expect_error(itis_facet(foo = "bar"),
    "didn't detect any facet. fields")

  expect_error(itis_facet(wt = "asdfaddf"),
    "wt must be one of")
})

#' Returns the review year for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame
#' @examples \dontrun{
#' review_year(tsn = 180541)
#' }
review_year <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getReviewYearFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- tc(parse_raw(out))
  tibble::as_tibble(
    tc(pick_cols(x, c("rankId","reviewYear","tsn")))
  )
}

#' Get global species completeness from tsn
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @examples \dontrun{
#' global_species_completeness(tsn = 180541)
#' global_species_completeness(180541, wt = "xml")
#' global_species_completeness(180541, wt = "json", raw = TRUE)
#' }
global_species_completeness <- function(tsn, wt = "json", raw = FALSE,
                                        ...) {

  out <- itis_GET("getGlobalSpeciesCompletenessFromTSN", list(tsn = tsn),
                  wt, ...)
  if (raw || wt == "xml") return(out)
  tibble::as_tibble(
    tc(pick_cols(parse_raw(out), c("completeness","rankId","tsn")))
  )
}

context("global_species_completeness")

skip_on_cran()

test_that("global_species_completeness tsn", {
  vcr::use_cassette("global_species_completeness", {
    aa <- global_species_completeness(tsn = 180541)
  })
  
  expect_is(aa, "data.frame")
  expect_is(aa$completeness, "character")
  expect_is(aa$rankid, "integer")
  expect_is(aa$tsn, "character")
})

test_that("global_species_completeness - xml works", {
  vcr::use_cassette("global_species_completeness-xml", {
    aa <- global_species_completeness(180541, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("global_species_completeness - raw JSON works", {
  vcr::use_cassette("global_species_completeness-json", {
    aa <- global_species_completeness(180541, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("global_species_completeness fails well", {
  expect_error(global_species_completeness(), "\"tsn\" is missing")

  expect_error(global_species_completeness("asdfafasffd", wt = "ffa"),
    "'wt' must be one of")

  vcr::use_cassette("global_species_completeness-errors", {
    expect_error(global_species_completeness("asdfadf"))
  })
})

#' Returns a list of the pulications used for the TSN.
#'
#' @export
#' @inheritParams accepted_names
#' @template tsn
#' @return a data.frame
#' @examples \dontrun{
#' publications(tsn = 70340)
#' publications(tsn = 70340, wt = "xml")
#'
#' publications(tsn = 70340, verbose = TRUE)
#' }
publications <- function(tsn, wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getPublicationsFromTSN", list(tsn = tsn), wt, ...)
  if (raw || wt == "xml") return(out)
  x <- parse_raw(out)$publications
  tibble::as_tibble(pick_cols(
    x,
    c("actualPubDate","isbn","issn","listedPubDate","pages",
      "pubComment","pubName","pubPlace","publisher","referenceAuthor",
      "referredTsn","title","updateDate")
  ))
}

context("publications")

test_that("publications basic functionality works", {
  vcr::use_cassette("publications", {
    aa <- publications(tsn = 70340)
  })

  expect_is(aa, "data.frame")
  expect_is(aa, "tbl_df")

  expect_is(aa$isbn, "character")
  expect_gt(NROW(aa), 0)
})

test_that("publications - xml works", {
  vcr::use_cassette("publications-xml", {
    aa <- publications(tsn = 70340, wt = "xml")
  })

  expect_is(aa, "character")
  expect_match(aa, "xmlns")
})

test_that("publications - raw JSON works", {
  vcr::use_cassette("publications-json", {
    aa <- publications(tsn = 70340, raw = TRUE)
  })
  
  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("publications fails well", {
  expect_error(publications(), "\"tsn\" is missing")
  expect_error(publications(tsn = 70340, wt = "ffa"), "'wt' must be one of")

  vcr::use_cassette("publications-fails-well", {
    # tsn's not found lead to 404
    expect_error(publications(tsn = "Asdfasdfa"),
      "Bad Request \\(HTTP 400\\)", class = "error")
  })
})

#' Get description of the ITIS service
#'
#' @export
#' @inheritParams accepted_names
#' @return a string, the ITIS web service description
#' @examples \dontrun{
#' description()
#' description(wt = "xml")
#' }
description <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getDescription", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$description
}

context("description")

skip_on_cran()

test_that("description basic functionality works", {
  vcr::use_cassette("description", {
    aa <- description()
  })
  
  expect_is(aa, "character")
  expect_match(aa, "ITIS Web Service")
})

test_that("description - xml works", {
  vcr::use_cassette("description-xml", {
    aa <- description(wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("description - raw JSON works", {
  vcr::use_cassette("description-json", {
    aa <- description(raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("description fails well", {
  expect_error(description(wt = "ffa"), "'wt' must be one of")
})

#' @title ritis
#' @description Interface to Integrated Taxonomic Information (ITIS)
#'
#' @section ritis package API:
#' All functions that start with `itis_` work with the ITIS Solr
#' API described at <https://www.itis.gov/solr_documentation.html>,
#' which uses the package \pkg{solrium}, and these functions have you
#' use the \pkg{solrium} function interfaces, so you can pass on parameters
#' to the \pkg{solrium} functions - so the \pkg{solrium} docs are important
#' here.
#'
#' All other functions work with the ITIS REST API described at
#' <https://www.itis.gov/ws_description.html>. For these methods,
#' they can grab data in either JSON or XML format. JSON is the default.
#' We parse the JSON to R native format, either data.frame, character
#' string, or list. You can get raw JSON as a character string back,
#' or raw XML as a character string, and then parse yourself with
#' \pkg{jsonlite} or \pkg{xml2}
#' 
#' You'll also be interested in the taxize book 
#' <https://taxize.dev/>
#' 
#' @section Terminology:
#' - "mononomial": a taxonomic name with one part, e.g, _Poa_
#' - "binomial": a taxonomic name with two parts, e.g, _Poa annua_
#' - "trinomial": a taxonomic name with three parts, e.g, _Poa annua annua_
#'
#' @name ritis-package
#' @aliases ritis
#' @docType package
#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}
NULL

#' List of fields that can be used in [solr] functions
#'
#' Each element in the list has a list of length tree, with:
#' 
#' - field: the field name, this is the name you can use in your 
#' queries
#' - definition: the definition of the field
#' - example: an example value
#'
#' @format A list of length 36
#' @source <https://www.itis.gov/solr_documentation.html>
#' @name solr_fields
#' @docType data
#' @keywords data
NULL

foo_terms <- function(path) {
  function(x, wt = "json", raw = FALSE, ...) {
    out <- itis_GET(path, list(srchKey = x), wt, ...)
    if (raw || wt == "xml") return(out)
    x <- parse_raw(out)
    if (NROW(x) == 0) return(tibble::tibble())
    if (is.null(x$itisTerms) || inherits(x$itisTerms, "logical")) {
      tibble::tibble()
    } else {
      dr_op(tibble::as_tibble(x$itisTerms), "class")
    }
  }
}
itisterms <- foo_terms("getITISTerms")
itisterms_com <- foo_terms("getITISTermsFromCommonName")
itisterms_sci <- foo_terms("getITISTermsFromScientificName")

#' Get ITIS terms, i.e., tsn's, authors, common names, and scientific names
#'
#' @export
#' @inheritParams accepted_names
#' @param query One or more common or scientific names, or partial names
#' @param what One of both (search common and scientific names), common
#' (search just common names), or scientific (search just scientific names)
#' @examples \dontrun{
#' # Get terms searching both common and scientific names
#' terms(query='bear')
#'
#' # Get terms searching just common names
#' terms(query='tarweed', "common")
#'
#' # Get terms searching just scientific names
#' terms(query='Poa annua', "scientific")
#'
#' # many at once
#' terms(query=c('Poa annua', 'Pinus contorta'), "scientific")
#' }
terms <- function(query, what = "both", wt = "json", raw = FALSE, ...) {
  what <- match.arg(what, c('both', 'scientific', 'common'))
  fun <- switch(what, both = itisterms, common = itisterms_com,
    scientific = itisterms_sci)
  temp <- lapply(query, fun, wt = wt, raw = raw, ...)
  if (length(query) == 1) return(temp[[1]])
  stats::setNames(temp, query)
}

context("terms")

skip_on_cran()

test_that("terms - both", {
  vcr::use_cassette("terms_both", {
    aa <- terms(query='brown bear')
  })
  
  expect_is(aa, "data.frame")
  expect_gt(NROW(aa), 1)
  expect_is(aa$author, "character")
  expect_is(aa$commonNames, "list")
  expect_is(aa$nameUsage, "character")
  expect_is(aa$scientificName, "character")
  expect_is(aa$tsn, "character")
})

test_that("terms - scientific", {
  vcr::use_cassette("terms_scientific", {
    aa <- terms(query='Poa annua', "scientific")
  })
  
  expect_is(aa, "data.frame")
  expect_gt(NROW(aa), 10)
  expect_is(aa$author, "character")
  expect_is(aa$commonNames, "list")
  expect_is(aa$nameUsage, "character")
  expect_is(aa$scientificName, "character")
  expect_is(aa$tsn, "character")
})

test_that("terms - scientific", {
  vcr::use_cassette("terms_common", {
    aa <- terms(query="hayfield tarweed", "common")
  })
  
  expect_is(aa, "data.frame")
  expect_gt(NROW(aa), 5)
  expect_is(aa$author, "character")
  expect_is(aa$commonNames, "list")
  expect_is(aa$nameUsage, "character")
  expect_is(aa$scientificName, "character")
  expect_is(aa$tsn, "character")
})

test_that("terms fails well", {
  expect_error(terms(), "\"query\" is missing")

  expect_error(terms("asdfafasffd", wt = "ffa"),
    "'wt' must be one of")
})

#' Provides a list of the unique languages used in the vernacular table.
#'
#' @export
#' @inheritParams accepted_names
#' @return a character vector of verncular names
#' @examples \dontrun{
#' vernacular_languages()
#' }
vernacular_languages <- function(wt = "json", raw = FALSE, ...) {
  out <- itis_GET("getVernacularLanguages", list(), wt, ...)
  if (raw || wt == "xml") return(out)
  parse_raw(out)$languageNames
}

context("usage")

skip_on_cran()

test_that("usage basic functionality works", {
  vcr::use_cassette("usage", {
    aa <- usage(tsn = 526852)
  })
  
  expect_is(aa, "data.frame")
  expect_equal(NROW(aa), 1)
  for (i in aa) expect_is(i, "character")
})

test_that("usage - xml works", {
  vcr::use_cassette("usage-xml", {
    aa <- usage(526852, wt = "xml")
  })

  expect_is(aa, "character")
  expect_true(grepl("xmlns", aa))
})

test_that("usage - raw JSON works", {
  vcr::use_cassette("usage-json", {
    aa <- usage(526852, raw = TRUE)
  })

  expect_is(aa, "character")
  expect_false(grepl("xmlns", aa))
})

test_that("usage fails well", {
  expect_error(usage(), "\"tsn\" is missing")

  expect_error(usage("asdfafasffd", wt = "ffa"),
    "'wt' must be one of")

  vcr::use_cassette("usage-errors", {
    expect_error(usage("asdfadf"))
  })
})

---
title: "ritis introduction"
author: "Scott Chamberlain"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ritis introduction}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, warning = FALSE, message = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse  =  TRUE,
  comment  =  "#>",
  warning = FALSE,
  message = FALSE,
  purl  =  NOT_CRAN,
  eval  =  NOT_CRAN
)
```

An interface to the Integrated Taxonomic Information System (ITIS)

See also the [taxize book](https://taxize.dev/) for 
a manual on working with taxonomic data in R, including with ITIS data.

## Installation

Install from CRAN

```{r eval=FALSE}
install.packages("ritis")
```

Or install the development version from GitHub

```{r installgh, eval=FALSE}
devtools::install_github("ropensci/ritis")
```

Load `ritis`

```{r}
library("ritis")
```

## ITIS Solr interface

There are four methods.

* `itis_search()` - Search
* `itis_group()` - Group
* `itis_highlight()` - Hightlight
* `itis_facet()` - Facet

These four methods use the equivalent functions in the package `solrium`, e.g.,
`ritis::itis_search()` uses `solrium::solr_search()`, etc. The `itis_*()` functions
simply use `...` to allow users to pass on parameters to the wrapped `solrium`
functions. So do read the `solrium` docs.

ITIS Solr API docs: <https://www.itis.gov/solr_documentation.html>

Some examples:

matches only monomials

```{r}
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/")
```

matches only binomials

```{r}
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")
```

## ITIS REST API interface

ITIS REST API docs: <http://www.itis.gov/ws_description.html>

The following are some example uses. There are many more methods not shown below

-------

Get accepted names for a TSN

```{r}
accepted_names(tsn = 504239)
```

Get common names for a TSN

```{r}
common_names(tsn = 183833)
```

Full hierarchy for a TSN

```{r}
hierarchy_full(tsn = 37906)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsn2lsid.R
\name{tsn2lsid}
\alias{tsn2lsid}
\title{Gets the unique LSID for the TSN, or an empty result if there is no match.}
\usage{
tsn2lsid(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a character string, an LSID, or \code{NULL} if nothing found
}
\description{
Gets the unique LSID for the TSN, or an empty result if there is no match.
}
\examples{
\dontrun{
tsn2lsid(tsn = 155166)
tsn2lsid(tsn = 333333333)
tsn2lsid(155166, raw = TRUE)
tsn2lsid(155166, wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itis_facet.R
\name{itis_facet}
\alias{itis_facet}
\title{ITIS Solr facet}
\usage{
itis_facet(..., proxy = NULL, callopts = list())
}
\arguments{
\item{...}{Arguments passed on to the \code{params} parameter of
the \code{\link[solrium:solr_facet]{solrium::solr_facet()}} function. See \link{solr_fields} for possible
parameters, and examples below}

\item{proxy}{List of arguments for a proxy connection,
including one or more of: url, port, username, password,
and auth. See \code{\link[crul:proxies]{crul::proxy()}} for  help, which is used to
construct the proxy connection.}

\item{callopts}{Curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
ITIS Solr facet
}
\examples{
\dontrun{
itis_facet(q = "rank:Species", rows = 0, facet.field = "kingdom")$facet_fields

x <- itis_facet(q = "hierarchySoFar:*$Aves$* AND rank:Species AND usage:valid",
   facet.pivot = "nameWInd,vernacular", facet.limit = -1, facet.mincount = 1,
   rows = 0)
head(x$facet_pivot$`nameWInd,vernacular`)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/description.R
\name{description}
\alias{description}
\title{Get description of the ITIS service}
\usage{
description(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a string, the ITIS web service description
}
\description{
Get description of the ITIS service
}
\examples{
\dontrun{
description()
description(wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_record.R
\name{full_record}
\alias{full_record}
\title{Get full record from TSN or lsid}
\usage{
full_record(tsn = NULL, lsid = NULL, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{lsid}{lsid for a taxonomic group (character)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get full record from TSN or lsid
}
\examples{
\dontrun{
# from tsn
full_record(tsn = 50423)
full_record(tsn = 202385)
full_record(tsn = 183833)

full_record(tsn = 183833, wt = "xml")
full_record(tsn = 183833, raw = TRUE)

# from lsid
full_record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
full_record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/other_sources.R
\name{other_sources}
\alias{other_sources}
\title{Returns a list of the other sources used for the TSN.}
\usage{
other_sources(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Returns a list of the other sources used for the TSN.
}
\examples{
\dontrun{
# results
other_sources(tsn=182662)
# no results
other_sources(tsn=2085272) 
# get xml
other_sources(tsn=182662, wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scientific_name.R
\name{scientific_name}
\alias{scientific_name}
\title{Returns the scientific name for the TSN. Also returns the component parts
(names and indicators) of the scientific name.}
\usage{
scientific_name(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Returns the scientific name for the TSN. Also returns the component parts
(names and indicators) of the scientific name.
}
\examples{
\dontrun{
scientific_name(tsn = 531894)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_any_match_paged.R
\name{search_any_match_paged}
\alias{search_any_match_paged}
\title{Search for any matched page}
\usage{
search_any_match_paged(
  x,
  pagesize = NULL,
  pagenum = NULL,
  ascend = NULL,
  wt = "json",
  raw = FALSE,
  ...
)
}
\arguments{
\item{x}{text or taxonomic serial number (TSN) (character or numeric)}

\item{pagesize}{An integer containing the page size (numeric)}

\item{pagenum}{An integer containing the page number (numeric)}

\item{ascend}{A boolean containing true for ascending sort order or false
for descending (logical)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame

a data.frame
}
\description{
Search for any matched page
}
\examples{
\dontrun{
search_any_match_paged(x=202385, pagesize=100, pagenum=1, ascend=FALSE)
search_any_match_paged(x="Zy", pagesize=100, pagenum=1, ascend=FALSE)
}
}
\seealso{
\code{\link{search_anymatch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accepted_names.R
\name{accepted_names}
\alias{accepted_names}
\title{Get accepted names from tsn}
\usage{
accepted_names(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
Zero row data.frame if the name is accepted, otherwise a data.frame
with information on the currently accepted name
}
\description{
Get accepted names from tsn
}
\examples{
\dontrun{
# TSN accepted - good name, empty data.frame returned
accepted_names(tsn = 208527)

# TSN not accepted - input TSN is old name, non-empty data.frame returned
accepted_names(tsn = 504239)

# raw json
accepted_names(tsn = 208527, raw = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currency.R
\name{currency}
\alias{currency}
\title{Get currency from tsn}
\usage{
currency(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Get currency from tsn
}
\examples{
\dontrun{
# currency data
currency(tsn=28727)
currency(tsn=28727, wt = "xml")
# no currency dat
currency(526852)
currency(526852, raw = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core_metadata.R
\name{core_metadata}
\alias{core_metadata}
\title{Get core metadata from tsn}
\usage{
core_metadata(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get core metadata from tsn
}
\examples{
\dontrun{
# coverage and currrency data
core_metadata(tsn=28727)
core_metadata(tsn=28727, wt = "xml")
# no coverage or currrency data
core_metadata(183671)
core_metadata(183671, wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ritis-package.R
\docType{package}
\name{ritis-package}
\alias{ritis-package}
\alias{ritis}
\title{ritis}
\description{
Interface to Integrated Taxonomic Information (ITIS)
}
\section{ritis package API}{

All functions that start with \code{itis_} work with the ITIS Solr
API described at \url{https://www.itis.gov/solr_documentation.html},
which uses the package \pkg{solrium}, and these functions have you
use the \pkg{solrium} function interfaces, so you can pass on parameters
to the \pkg{solrium} functions - so the \pkg{solrium} docs are important
here.

All other functions work with the ITIS REST API described at
\url{https://www.itis.gov/ws_description.html}. For these methods,
they can grab data in either JSON or XML format. JSON is the default.
We parse the JSON to R native format, either data.frame, character
string, or list. You can get raw JSON as a character string back,
or raw XML as a character string, and then parse yourself with
\pkg{jsonlite} or \pkg{xml2}

You'll also be interested in the taxize book
\url{https://taxize.dev/}
}

\section{Terminology}{

\itemize{
\item "mononomial": a taxonomic name with one part, e.g, \emph{Poa}
\item "binomial": a taxonomic name with two parts, e.g, \emph{Poa annua}
\item "trinomial": a taxonomic name with three parts, e.g, \emph{Poa annua annua}
}
}

\author{
Scott Chamberlain \email{myrmecocystus@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchy.R
\name{hierarchy}
\alias{hierarchy}
\alias{hierarchy_down}
\alias{hierarchy_up}
\alias{hierarchy_full}
\title{Get hierarchy down from tsn}
\usage{
hierarchy_down(tsn, wt = "json", raw = FALSE, ...)

hierarchy_up(tsn, wt = "json", raw = FALSE, ...)

hierarchy_full(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get hierarchy down from tsn
}
\details{
Hierarchy methods:
\itemize{
\item hierarchy_down: Get hierarchy down from tsn
\item hierarchy_up: Get hierarchy up from tsn
\item hierarchy_full: Get full hierarchy from tsn
}
}
\examples{
\dontrun{
## Full down (class Mammalia)
hierarchy_down(tsn=179913)

## Full up (genus Agoseris)
hierarchy_up(tsn=36485)

## Full hierarchy
### genus Liatris
hierarchy_full(tsn=37906)
### get raw data back
hierarchy_full(tsn=37906, raw = TRUE)
### genus Baetis, get xml back
hierarchy_full(100800, wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsid2tsn.R
\name{lsid2tsn}
\alias{lsid2tsn}
\title{Gets the TSN corresponding to the LSID, or an empty result if there is no match.}
\usage{
lsid2tsn(lsid, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{lsid}{(character) lsid for a taxonomic group. Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Gets the TSN corresponding to the LSID, or an empty result if there is no match.
}
\examples{
\dontrun{
lsid2tsn(lsid="urn:lsid:itis.gov:itis_tsn:28726")
lsid2tsn(lsid="urn:lsid:itis.gov:itis_tsn:28726", wt = "xml")
lsid2tsn("urn:lsid:itis.gov:itis_tsn:0")
lsid2tsn("urn:lsid:itis.gov:itis_tsn:0", wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/review_year.R
\name{review_year}
\alias{review_year}
\title{Returns the review year for the TSN.}
\usage{
review_year(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Returns the review year for the TSN.
}
\examples{
\dontrun{
review_year(tsn = 180541)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itis_search.R
\name{itis_search}
\alias{itis_search}
\title{ITIS Solr search}
\usage{
itis_search(..., proxy = NULL, callopts = list())
}
\arguments{
\item{...}{Arguments passed on to the \code{params} parameter of
the \code{\link[solrium:solr_search]{solrium::solr_search()}} function. See \link{solr_fields} for possible
parameters, and examples below}

\item{proxy}{List of arguments for a proxy connection,
including one or more of: url, port, username, password,
and auth. See \code{\link[crul:proxies]{crul::proxy()}} for  help, which is used to
construct the proxy connection.}

\item{callopts}{Curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
ITIS Solr search
}
\details{
The syntax for this function can be a bit hard to grasp. See
https://itis.gov/solr_examples.html for help on generating the
syntax ITIS wants for specific searches.
}
\examples{
\dontrun{
itis_search(q = "tsn:182662")

# get all orders within class Aves (birds)
z <- itis_search(q = "rank:Class AND nameWOInd:Aves")
hierarchy_down(z$tsn)

# get taxa "downstream" from a target taxon
## taxize and taxizedb packages have downstream() fxns, but
## you can do a similar thing here by iteratively drilling down
## the taxonomic hierarchy
## here, we get families within Aves
library(data.table)
aves <- itis_search(q = "rank:Class AND nameWOInd:Aves")
aves_orders <- hierarchy_down(aves$tsn)
aves_families <- lapply(aves_orders$tsn, hierarchy_down)
rbindlist(aves_families)

# the tila operator
itis_search(q = "nameWOInd:Liquidamber\\\\ styraciflua~0.4")

# matches only monomials
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/")

# matches only binomials
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")

# matches only trinomials
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*/")

# matches binomials or trinomials
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{1,1}[A-Za-z0-9]*[ ]{0,1}[A-Za-z0-9]*/")

itis_search(q = "nameWOInd:Poa\\\\ annua")

# pagination
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/", rows = 2)
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/", rows = 200)

# select fields to return
itis_search(q = "nameWOInd:/[A-Za-z0-9]*[ ]{0,0}*/",
   fl = c('nameWInd', 'tsn'))
}
}
\references{
\url{https://www.itis.gov/solr_documentation.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_scientific.R
\name{search_scientific}
\alias{search_scientific}
\title{Search by scientific name}
\usage{
search_scientific(x, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{x}{text or taxonomic serial number (TSN) (character or numeric)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Search by scientific name
}
\examples{
\dontrun{
search_scientific("Tardigrada")
search_scientific("Quercus douglasii")
}
}
\seealso{
\code{\link{search_common}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solr.R
\name{solr}
\alias{solr}
\title{ITIS Solr Methods}
\description{
ITIS provides access to their data via their Solr service described at
\url{https://www.itis.gov/solr_documentation.html}.  This is a powerful
interace to ITIS data as you have access to a very flexible query
interface.
}
\details{
See \link{solr_fields} and \url{https://www.itis.gov/solr_documentation.html}
for guidance on available fields.
}
\section{Functions}{

\itemize{
\item \code{\link[=itis_search]{itis_search()}} - Search
\item \code{\link[=itis_group]{itis_group()}} - Group
\item \code{\link[=itis_highlight]{itis_highlight()}} - Highlight
\item \code{\link[=itis_facet]{itis_facet()}} - Facet
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank_names.R
\name{rank_names}
\alias{rank_names}
\title{Provides a list of all the unique rank names contained in the database and
their kingdom and rank ID values.}
\usage{
rank_names(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame, with columns:
\itemize{
\item kingdomname
\item rankid
\item rankname
}
}
\description{
Provides a list of all the unique rank names contained in the database and
their kingdom and rank ID values.
}
\examples{
\dontrun{
rank_names()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jurisdiction.R
\name{jurisdiction}
\alias{jurisdiction}
\alias{jurisdictional_origin}
\alias{jurisdiction_origin_values}
\alias{jurisdiction_values}
\title{Get jurisdictional origin from tsn}
\usage{
jurisdictional_origin(tsn, wt = "json", raw = FALSE, ...)

jurisdiction_origin_values(wt = "json", raw = FALSE, ...)

jurisdiction_values(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
\itemize{
\item jurisdictional_origin: data.frame
\item jurisdiction_origin_values: data.frame
\item jurisdiction_values: character vector
}
}
\description{
Get jurisdictional origin from tsn
}
\details{
Jurisdiction methods:
\itemize{
\item jurisdictional_origin: Get jurisdictional origin from tsn
\item jurisdiction_origin_values: Get jurisdiction origin values
\item jurisdiction_values: Get all possible jurisdiction values
}
}
\examples{
\dontrun{
jurisdictional_origin(tsn=180543)
jurisdictional_origin(tsn=180543, wt = "xml")

jurisdiction_origin_values()

jurisdiction_values()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ritis-package.R
\docType{data}
\name{solr_fields}
\alias{solr_fields}
\title{List of fields that can be used in \link{solr} functions}
\format{
A list of length 36
}
\source{
\url{https://www.itis.gov/solr_documentation.html}
}
\description{
Each element in the list has a list of length tree, with:
}
\details{
\itemize{
\item field: the field name, this is the name you can use in your
queries
\item definition: the definition of the field
\item example: an example value
}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/last_change_date.R
\name{last_change_date}
\alias{last_change_date}
\title{Provides the date the ITIS database was last updated}
\usage{
last_change_date(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
character value with a date
}
\description{
Provides the date the ITIS database was last updated
}
\examples{
\dontrun{
last_change_date()
last_change_date(wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itis_highlight.R
\name{itis_highlight}
\alias{itis_highlight}
\title{ITIS Solr highlight}
\usage{
itis_highlight(..., proxy = NULL, callopts = list())
}
\arguments{
\item{...}{Arguments passed on to the \code{params} parameter of
the \code{\link[solrium:solr_highlight]{solrium::solr_highlight()}} function. See \link{solr_fields} for possible
parameters, and examples below}

\item{proxy}{List of arguments for a proxy connection,
including one or more of: url, port, username, password,
and auth. See \code{\link[crul:proxies]{crul::proxy()}} for  help, which is used to
construct the proxy connection.}

\item{callopts}{Curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
ITIS Solr highlight
}
\examples{
\dontrun{
itis_highlight(q = "rank:Species", hl.fl = 'rank', rows=10)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/record.R
\name{record}
\alias{record}
\title{Gets a record from an LSID}
\usage{
record(lsid, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{lsid}{lsid for a taxonomic group (character). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Gets a record from an LSID
}
\details{
Gets the partial ITIS record for the TSN in the LSID, found by comparing the
TSN in the search key to the TSN field. Returns an empty result set if
there is no match or the TSN is invalid.
}
\examples{
\dontrun{
record(lsid = "urn:lsid:itis.gov:itis_tsn:180543")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_common.R
\name{search_common}
\alias{search_common}
\title{Search for tsn by common name}
\usage{
search_common(x, from = "all", wt = "json", raw = FALSE, ...)
}
\arguments{
\item{x}{text or taxonomic serial number (TSN) (character or numeric)}

\item{from}{(character) One of "all", "begin", or "end". See Details.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Search for tsn by common name
}
\details{
The \code{from} parameter:
\itemize{
\item all - Search against the \code{searchByCommonName} API route, which
searches entire name string
\item begin - Search against the \code{searchByCommonNameBeginsWith} API
route, which searches for a match at the beginning of a name string
\item end - Search against the \code{searchByCommonNameEndsWith} API route,
which searches for a match at the end of a name string
}
}
\examples{
\dontrun{
search_common("american bullfrog")
search_common("ferret-badger")
search_common("polar bear")

# comparison: all, begin, end
search_common("inch")
search_common("inch", from = "begin")
search_common("inch", from = "end")

# end
search_common("snake", from = "end")
}
}
\seealso{
\code{\link[=search_scientific]{search_scientific()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/any_match_count.R
\name{any_match_count}
\alias{any_match_count}
\title{Get any match count.}
\usage{
any_match_count(x, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{x}{text or taxonomic serial number (TSN) (character or numeric)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
An integer containing the number of matches the search will return.
}
\description{
Get any match count.
}
\examples{
\dontrun{
any_match_count(x = 202385)
any_match_count(x = "dolphin")
any_match_count(x = "dolphin", wt = "xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/synonym_names.R
\name{synonym_names}
\alias{synonym_names}
\title{Returns a list of the synonyms (if any) for the TSN.}
\usage{
synonym_names(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Returns a list of the synonyms (if any) for the TSN.
}
\examples{
\dontrun{
synonym_names(tsn=183671) # tsn not accepted
synonym_names(tsn=526852) # tsn accepted
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_data.R
\name{date_data}
\alias{date_data}
\title{Get date data from tsn}
\usage{
date_data(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get date data from tsn
}
\examples{
\dontrun{
date_data(tsn = 180543)
date_data(180543, wt = "xml")
date_data(180543, wt = "json", raw = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geographic_values.R
\name{geographic_values}
\alias{geographic_values}
\title{Get all possible geographic values}
\usage{
geographic_values(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
character vector of geographic names
}
\description{
Get all possible geographic values
}
\examples{
\dontrun{
geographic_values()
geographic_values(wt = "xml")
geographic_values(wt = "json", raw = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vernacular_languages.R
\name{vernacular_languages}
\alias{vernacular_languages}
\title{Provides a list of the unique languages used in the vernacular table.}
\usage{
vernacular_languages(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a character vector of verncular names
}
\description{
Provides a list of the unique languages used in the vernacular table.
}
\examples{
\dontrun{
vernacular_languages()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terms.R
\name{terms}
\alias{terms}
\title{Get ITIS terms, i.e., tsn's, authors, common names, and scientific names}
\usage{
terms(query, what = "both", wt = "json", raw = FALSE, ...)
}
\arguments{
\item{query}{One or more common or scientific names, or partial names}

\item{what}{One of both (search common and scientific names), common
(search just common names), or scientific (search just scientific names)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get ITIS terms, i.e., tsn's, authors, common names, and scientific names
}
\examples{
\dontrun{
# Get terms searching both common and scientific names
terms(query='bear')

# Get terms searching just common names
terms(query='tarweed', "common")

# Get terms searching just scientific names
terms(query='Poa annua', "scientific")

# many at once
terms(query=c('Poa annua', 'Pinus contorta'), "scientific")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/experts.R
\name{experts}
\alias{experts}
\title{Get expert information for the TSN.}
\usage{
experts(tsn, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get expert information for the TSN.
}
\examples{
\dontrun{
experts(tsn = 180544)
experts(180544, wt = "xml")
experts(180544, raw = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsn_by_vernacular_language.R
\name{tsn_by_vernacular_language}
\alias{tsn_by_vernacular_language}
\title{Get tsn by vernacular language}
\usage{
tsn_by_vernacular_language(language, wt = "json", raw = FALSE, ...)
}
\arguments{
\item{language}{A string containing the language. This is a language string,
not the international language code (character)}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Get tsn by vernacular language
}
\examples{
\dontrun{
tsn_by_vernacular_language(language = "french")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kingdoms.R
\name{kingdoms}
\alias{kingdoms}
\alias{kingdom_name}
\alias{kingdom_names}
\title{Get kingdom names from tsn}
\usage{
kingdom_name(tsn, wt = "json", raw = FALSE, ...)

kingdom_names(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\description{
Get kingdom names from tsn
}
\details{
\itemize{
\item kingdom_name: Get kingdom name for a TSN
\item kingdom_names: Get all possible kingdom names
}
}
\examples{
\dontrun{
kingdom_name(202385)
kingdom_name(202385, wt = "xml")
kingdom_names()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/credibility.R
\name{credibility}
\alias{credibility}
\alias{credibility_rating}
\alias{credibility_ratings}
\title{Get credibility rating from tsn}
\usage{
credibility_rating(tsn, wt = "json", raw = FALSE, ...)

credibility_ratings(wt = "json", raw = FALSE, ...)
}
\arguments{
\item{tsn}{TSN for a taxonomic group (numeric). Required.}

\item{wt}{(character) One of "json" or "xml". Required.}

\item{raw}{(logical) Return raw JSON or XML as character string. Required.
Default: \code{FALSE}}

\item{...}{curl options passed on to \link[crul:HttpClient]{crul::HttpClient}}
}
\value{
a data.frame
}
\description{
Get credibility rating from tsn
}
\details{
methods:
\itemize{
\item credibility_rating: Get credibility rating for a tsn
\item credibility_ratings: Get possible credibility ratings
}
}
\examples{
\dontrun{
credibility_rating(tsn = 526852)
credibility_rating(526852, wt = "xml")
credibility_rating(526852, raw = TRUE)

credibility_ratings()
credibility_ratings(wt = "xml")
credibility_ratings(raw = TRUE)
}
}
